@@ -47,6 +47,7 @@
 #pod = L<Dist::Zilla>
 #pod = L<Dist::Zilla::Role>
 #pod = L<Dist::Zilla::Role::Plugin>
+#pod = L<Dist::Zilla::Role::FileInjector>
 #pod = L<Dist::Zilla::Role::FileGatherer>
 #pod = L<Dist::Zilla::Plugin::Manifest>
 #pod = L<Dist::Zilla::Plugin::Manifest::Write::Manual>
@@ -63,13 +64,15 @@ use namespace::autoclean;
 
 # PODNAME: Dist::Zilla::Plugin::Manifest::Write
 # ABSTRACT: Have annotated MANIFEST in your distribution
-our $VERSION = '0.005'; # VERSION
+our $VERSION = '0.006'; # VERSION
 
 with 'Dist::Zilla::Role::FileGatherer';
+with 'Dist::Zilla::Role::BeforeBuild';
 with 'Dist::Zilla::Role::ErrorLogger';
 
 use Dist::Zilla::File::FromCode;
 use List::Util;
+use Try::Tiny;
 
 use Dist::Zilla::Role::File 5.023 ();       # Hint for `AutoPrereqs`.
     #   We do not consume the role but just require the specified version. Before this version
@@ -77,6 +80,64 @@ use Dist::Zilla::Role::File 5.023 ();       # Hint for `AutoPrereqs`.
 
 # --------------------------------------------------------------------------------------------------
 
+#pod =head1 FUNCTIONS
+#pod
+#pod I would expect to find these functions in C<Dist::Zilla>. Actually, C<Dist::Zilla::Util> defines
+#pod the function C<expand_config_package_name>, but that function "is likely to change or go away" and
+#pod there is no reverse transformation.
+#pod
+#pod =cut
+
+# --------------------------------------------------------------------------------------------------
+
+#pod =func __plugin_name
+#pod
+#pod     $str = __plugin_name( 'Dist::Zilla::Plugin::Name' );    # 'Name'
+#pod     $str = __plugin_name( $plugin );
+#pod
+#pod The function takes either reference to a plugin object or a string, package name, and returns
+#pod C<Dist::Zilla> plugin name: If its package name begins with C<Dist::Zilla::Plugin::>, this common
+#pod prefix is dropped, otherwise the package name prepended with C<=>.
+#pod
+#pod =cut
+
+sub __plugin_name($) {      ## no critic ( ProhibitSubroutinePrototypes )
+    my ( $arg ) = @_;
+    my $blessed = blessed( $arg );
+    if ( $blessed ) {
+        $arg = $blessed;
+    };
+    $arg = ( $arg =~ s{^Dist::Zilla::Plugin::}{} ? '' : '=' ) . $arg;
+    return $arg;
+};
+
+# --------------------------------------------------------------------------------------------------
+
+#pod =func __package_name
+#pod
+#pod     $str = __package_name( 'Name' );  # 'Dist::Zilla::Plugin::Name'
+#pod     $str = __package_name( $plugin );
+#pod
+#pod This is operation opposite to C<__plugin_name>. It takes either reference to plugin object, or
+#pod string, plugin name, and returns package name.
+#pod
+#pod This function is similar to C<expand_config_package_name> from C<Dist::Zilla::Util>, with minor
+#pod difference: this function works with plugins only (not with plugin bundles and stashes), and
+#pod accepts also reference to plugin objects.
+#pod
+#pod =cut
+
+sub __package_name($) {     ## no critic ( ProhibitSubroutinePrototypes )
+    my ( $arg ) = @_;
+    my $blessed = blessed( $arg );
+    if ( $blessed ) {
+        return $blessed;
+    };
+    return ( $arg =~ s{^=}{} ? '' : 'Dist::Zilla::Plugin::' ) . $arg;
+};
+
+# --------------------------------------------------------------------------------------------------
+
 #pod =Method BUILDARGS
 #pod
 #pod The method splits C<source_providers> and C<metainfo_providers> options. It allows user to specify
@@ -101,11 +162,30 @@ around BUILDARGS => sub {
 
 # --------------------------------------------------------------------------------------------------
 
+#pod =attr strict
+#pod
+#pod Strictness of checking source and metainfo provider names: -1 (no checks), 0 (some mistakes are
+#pod fatal, some are not), or 1 (all mistakes are fatal).
+#pod
+#pod C<Int>, read-only. Default is 1.
+#pod
+#pod See L<Dist::Zilla::Plugin::Manifets::Write::Manual/"strict">.
+#pod
+#pod =cut
+
+has strict => (
+    is      => 'ro',
+    isa     => 'Int',
+    default => 2,
+);
+
+# --------------------------------------------------------------------------------------------------
+
 #pod =attr manifest
 #pod
 #pod Name of manifest file to write.
 #pod
-#pod Str, read-only. Default value is C<'MANIFEST'>.
+#pod C<Str>, read-only. Default value is C<'MANIFEST'>.
 #pod
 #pod =cut
 
@@ -121,8 +201,8 @@ has manifest => (
 #pod
 #pod Array of source providers.
 #pod
-#pod ArrayRef[Str], read-only, default value is empty array. Init argument (and configure file option)
-#pod name is C<source_providers>.
+#pod C<ArrayRef[Str]>, read-only, default value is empty array. Init argument (and configure file
+#pod option) name is C<source_providers>.
 #pod
 #pod =cut
 
@@ -145,8 +225,8 @@ sub _build_source_list {
 #pod
 #pod Array of metainfo providers.
 #pod
-#pod ArrayRef[Str], read-only, default value is empty array. Init argument (and configure file option)
-#pod name is C<metainfo_providers>.
+#pod C<ArrayRef[Str]>, read-only, default value is empty array. Init argument (and configure file
+#pod option) name is C<metainfo_providers>.
 #pod
 #pod =cut
 
@@ -161,12 +241,11 @@ has metainfo_list => (
 
 sub _build_metainfo_list {
     my ( $self ) = @_;
-    ref( $self ) =~ m{\ADist::Zilla::Plugin::(\S+)\z} or die;
-    my $this = $1;
-    return [
-        qw{ MetaYAML MetaJSON Manifest Manifest::Write },
-        $this eq 'Manifest::Write' ? () : $this
-    ];
+    my @list = ( qw{ MetaYAML MetaJSON Manifest Manifest::Write }, __plugin_name( $self ) );
+    if ( $self->strict >= 0 ) {
+        @list = grep( $self->_does_role( $_, 'FileInjector' ), @list );
+    };
+    return \@list;
 };
 
 # --------------------------------------------------------------------------------------------------
@@ -176,7 +255,7 @@ sub _build_metainfo_list {
 #pod Hash of source providers (plugin name => boolean), lazily initialized from C<source_list>.
 #pod Introduced to make check "is plugin a source provider?" easier.
 #pod
-#pod HashRef[Str], read-only, not an init arg.
+#pod C<HashRef[Str]>, read-only, not an init arg.
 #pod
 #pod =cut
 
@@ -213,7 +292,8 @@ has metainfo_hash => (
 
 sub _build_metainfo_hash {
     my ( $self ) = @_;
-    return { map( ( $_ => 1 ), @{ $self->metainfo_list } ) };
+    my %hash = map( ( $_ => 1 ), @{ $self->metainfo_list } );
+    return \%hash;
 };
 
 # --------------------------------------------------------------------------------------------------
@@ -225,7 +305,7 @@ sub _build_metainfo_hash {
 #pod This attribute maps internal file deed identifiers (C<source>, C<metainfo>, C<other>) to
 #pod user-visible names used in manifest (project name, C<metainfo>, and C<3rd party> respectively).
 #pod
-#pod HashRef, read-only.
+#pod C<HashRef>, read-only.
 #pod
 #pod =cut
 
@@ -252,7 +332,7 @@ sub _build_file_deeds {
 #pod
 #pod Width of file deed column. Computed from file deed visible names.
 #pod
-#pod Int, read-only.
+#pod C<Int>, read-only.
 #pod
 #pod =cut
 
@@ -271,6 +351,36 @@ sub _build_file_deed_width {
 
 # --------------------------------------------------------------------------------------------------
 
+#pod =method before_build
+#pod
+#pod The method checks validity of source and metainfo provider names.
+#pod
+#pod =cut
+
+sub before_build {
+    my ( $self ) = @_;
+    if ( $self->strict >= 0 ) {
+        my $log_warning = $self->strict > 0 ? 'log_error' : 'log';
+        my $zilla = $self->zilla;
+        for my $provider ( @{ $self->source_list }, @{ $self->metainfo_list } ) {
+            if ( $self->_does_role( $provider, 'Plugin' ) ) {
+                if ( $self->_does_role( $provider, 'FileInjector' ) ) {
+                    # ok.
+                } else {
+                    #   It is a plugin but not a `FileInjector`.
+                    $self->log_error( [ '%s is not a FileInjector', $provider ] );
+                };
+            } else {
+                #   It is not a plugin or plugin, but not loaded.
+                $self->$log_warning( [ '%s is not a loaded plugin', $provider ] );
+            };
+        };
+        $self->abort_if_error();
+    };
+};
+
+# --------------------------------------------------------------------------------------------------
+
 #pod =method gather_files
 #pod
 #pod This is the main method of the class. It adds a file with name C<< $self->manifest >> to the
@@ -375,7 +485,6 @@ sub _comment {
 #pod     content added by COPYING (Dist::Zilla::Plugin::GenerateFile line 114)
 #pod     filename set by GatherFromManifest (Dist::Zilla::Plugin::GatherFromManifest line 125)
 #pod     encoded_content added by GatherFromManifest (Dist::Zilla::Plugin::GatherFromManifest line 126)
-#pod     bytes from coderef added by VdB::WriteManifest (Dist::Zilla::Plugin::VdB::WriteManifest line 94)
 #pod     text from coderef added by MetaJSON (Dist::Zilla::Plugin::MetaJSON line 83)
 #pod     content set by TemplateFiles (Dist::Zilla::Plugin::TemplateFiles line 35)
 #pod     content set by OurPkgVersion (Dist::Zilla::Plugin::OurPkgVersion line 82)
@@ -428,12 +537,8 @@ sub _file_history {
 
 sub _plugin {
     my ( $self, $entry ) = @_;
-    my $plugin = $entry->{ package };
-    $plugin =~ s{\ADist::Zilla::Plugin::}{} or do {
-        $plugin = '=' . $plugin;
-    };
-    return $plugin;
-}
+    return __plugin_name( $entry->{ package } );
+};
 
 # --------------------------------------------------------------------------------------------------
 
@@ -518,6 +623,20 @@ around mvp_multivalue_args => sub {
 
 # --------------------------------------------------------------------------------------------------
 
+sub _does_role {
+    my ( $self, $plugin, $role ) = @_;
+    my $package = __package_name( $plugin );
+    my $ok;
+    try {
+        $ok = $package->meta->does_role( 'Dist::Zilla::Role::' . $role );
+    } catch {
+        $self->log_debug( "$_" );
+    };
+    return $ok;
+};
+
+# --------------------------------------------------------------------------------------------------
+
 __PACKAGE__->meta->make_immutable;
 
 1;
@@ -577,7 +696,7 @@ Dist::Zilla::Plugin::Manifest::Write - Have annotated MANIFEST in your distribut
 
 =head1 VERSION
 
-Version 0.005, released on 2015-07-15 20:47 UTC.
+Version 0.006, released on 2015-07-23 19:11 UTC.
 
 =head1 WHAT?
 
@@ -634,32 +753,41 @@ multiple plugin names in one line in F<dist.ini> file, e. g.:
 
 =head1 OBJECT ATTRIBUTES
 
+=head2 strict
+
+Strictness of checking source and metainfo provider names: -1 (no checks), 0 (some mistakes are
+fatal, some are not), or 1 (all mistakes are fatal).
+
+C<Int>, read-only. Default is 1.
+
+See L<Dist::Zilla::Plugin::Manifets::Write::Manual/"strict">.
+
 =head2 manifest
 
 Name of manifest file to write.
 
-Str, read-only. Default value is C<'MANIFEST'>.
+C<Str>, read-only. Default value is C<'MANIFEST'>.
 
 =head2 source_list
 
 Array of source providers.
 
-ArrayRef[Str], read-only, default value is empty array. Init argument (and configure file option)
-name is C<source_providers>.
+C<ArrayRef[Str]>, read-only, default value is empty array. Init argument (and configure file
+option) name is C<source_providers>.
 
 =head2 metainfo_list
 
 Array of metainfo providers.
 
-ArrayRef[Str], read-only, default value is empty array. Init argument (and configure file option)
-name is C<metainfo_providers>.
+C<ArrayRef[Str]>, read-only, default value is empty array. Init argument (and configure file
+option) name is C<metainfo_providers>.
 
 =head2 source_hash
 
 Hash of source providers (plugin name => boolean), lazily initialized from C<source_list>.
 Introduced to make check "is plugin a source provider?" easier.
 
-HashRef[Str], read-only, not an init arg.
+C<HashRef[Str]>, read-only, not an init arg.
 
 =head2 metainfo_hash
 
@@ -672,16 +800,20 @@ Like C<source_hash>, but for metainfo providers.
 This attribute maps internal file deed identifiers (C<source>, C<metainfo>, C<other>) to
 user-visible names used in manifest (project name, C<metainfo>, and C<3rd party> respectively).
 
-HashRef, read-only.
+C<HashRef>, read-only.
 
 =head2 file_deed_width
 
 Width of file deed column. Computed from file deed visible names.
 
-Int, read-only.
+C<Int>, read-only.
 
 =head1 OBJECT METHODS
 
+=head2 before_build
+
+The method checks validity of source and metainfo provider names.
+
 =head2 gather_files
 
 This is the main method of the class. It adds a file with name C<< $self->manifest >> to the
@@ -706,7 +838,6 @@ are few examples:
     content added by COPYING (Dist::Zilla::Plugin::GenerateFile line 114)
     filename set by GatherFromManifest (Dist::Zilla::Plugin::GatherFromManifest line 125)
     encoded_content added by GatherFromManifest (Dist::Zilla::Plugin::GatherFromManifest line 126)
-    bytes from coderef added by VdB::WriteManifest (Dist::Zilla::Plugin::VdB::WriteManifest line 94)
     text from coderef added by MetaJSON (Dist::Zilla::Plugin::MetaJSON line 83)
     content set by TemplateFiles (Dist::Zilla::Plugin::TemplateFiles line 35)
     content set by OurPkgVersion (Dist::Zilla::Plugin::OurPkgVersion line 82)
@@ -756,6 +887,33 @@ C<added> to distribution, otherwise file C<built>.
 This method tells C<Dist::Zilla> that C<source_providers> and C<metainfo_providers> are multi-value
 options (i. e. can be specified in F<dist.ini> several times).
 
+=head1 FUNCTIONS
+
+I would expect to find these functions in C<Dist::Zilla>. Actually, C<Dist::Zilla::Util> defines
+the function C<expand_config_package_name>, but that function "is likely to change or go away" and
+there is no reverse transformation.
+
+=head2 __plugin_name
+
+    $str = __plugin_name( 'Dist::Zilla::Plugin::Name' );    # 'Name'
+    $str = __plugin_name( $plugin );
+
+The function takes either reference to a plugin object or a string, package name, and returns
+C<Dist::Zilla> plugin name: If its package name begins with C<Dist::Zilla::Plugin::>, this common
+prefix is dropped, otherwise the package name prepended with C<=>.
+
+=head2 __package_name
+
+    $str = __package_name( 'Name' );  # 'Dist::Zilla::Plugin::Name'
+    $str = __package_name( $plugin );
+
+This is operation opposite to C<__plugin_name>. It takes either reference to plugin object, or
+string, plugin name, and returns package name.
+
+This function is similar to C<expand_config_package_name> from C<Dist::Zilla::Util>, with minor
+difference: this function works with plugins only (not with plugin bundles and stashes), and
+accepts also reference to plugin objects.
+
 =head1 SEE ALSO
 
 =over 4
@@ -766,6 +924,8 @@ options (i. e. can be specified in F<dist.ini> several times).
 
 =item L<Dist::Zilla::Role::Plugin>
 
+=item L<Dist::Zilla::Role::FileInjector>
+
 =item L<Dist::Zilla::Role::FileGatherer>
 
 =item L<Dist::Zilla::Plugin::Manifest>
