@@ -11,13 +11,13 @@ package CustomPlugin;
 use Moose;
 use namespace::autoclean;
 
-extends 'Dist::Zilla::Plugin::InlineFiles';
+with 'Dist::Zilla::Role::FileGatherer';
 
 use Dist::Zilla::File::FromCode;
 use Dist::Zilla::File::InMemory;
 
-after gather_files => sub {
-    my ( $self) = @_;
+sub gather_files {
+    my ( $self ) = @_;
     $self->add_file( Dist::Zilla::File::FromCode->new(
         name => 'GeneratedFile.txt',
         code_return_type => 'bytes',
@@ -38,10 +38,4 @@ after gather_files => sub {
 
 1;
 
-__DATA__
-__[lib/InlineModule.pm]__
-package InlineModule;
-1;
-__END__
-
 # end of file #
